# #!/usr/bin/env python
# # coding=utf-8
# """
# @athor:weifeng.guo
# @data:2019/3/26 13:56
# @filename:flask_reset_client
# """
# # pylint: disable=wrong-import-position, relative-import
# import sys
# import os
# import re
# import argparse
# import platform
# sys.path.append(os.path.abspath(os.path.join(os.getcwd(), "..", "..", "..")))
# from rest_client import RestClient
#
#
# def get_function_list(file_path):
#     func_list = list()
#     class_name = None
#     for line in open(file_path).readlines():
#         if r"class Test" in line:
#             class_name = re.findall("class ([a-zA-Z_]+)", line)[0]
#
#         if r"    def test_" in line and class_name:
#             test_name = re.findall("def ([a-zA-Z0-9_]+)", line)[0]
#             func_list.append("{}:{}.{}".format(file_path, class_name, test_name))
#
#     return func_list
#
#
# def list_test_case(file_list, keyword=None):
#     for file_path in file_list:
#         if keyword:
#             if keyword in file_path:
#                 if platform.system() == "Linux":
#                     print(file_path.replace(keyword, "\033[1;31m{}\033[0m".format(keyword)))
#                 else:
#                     print(file_path)
#
#             for func_name in get_function_list(file_path):
#                 if keyword in func_name:
#                     if platform.system() == "Linux":
#                         print(func_name.replace(keyword, "\033[1;31m{}\033[0m".format(keyword)))
#                     else:
#                         print(func_name)
#         else:
#             print(file_path)
#
#
# def run(args):
#     rest_client = RestClient(args.targetip)
#     if args.list:
#         if args.cmd == "all":
#             result_list = rest_client.test_case.list(args.cmd)
#             list_test_case(result_list)
#         else:
#             result_list = rest_client.test_case.list(args.cmd)
#             list_test_case(result_list)
#     elif args.nose:
#         print(rest_client.test_case.run(None, ip=args.targetip, command=args.cmd,
#                                         loglevel=args.loglevel, controller=args.controller,
#                                         flash=args.flashtype, extra=args.extra,
#                                         nose=args.nose)['msg'])
#     elif args.regress:
#         print(rest_client.test_case.run(None, ip=args.targetip, command=args.cmd,
#                                         loglevel=args.loglevel, controller=args.controller,
#                                         flash=args.flashtype, extra=args.extra,
#                                         regress=args.regress)['msg'])
#     else:
#         print(rest_client.test_case.run(None, ip=args.targetip, command=args.cmd,
#                                         loglevel=args.loglevel, controller=args.controller,
#                                         flash=args.flashtype, extra=args.extra,
#                                         python=args.python)['msg'])
#
#
# if __name__ == '__main__':
#     PARSER = argparse.ArgumentParser(description='Restful Client')
#     PARSER.add_argument('-t', '--targetip', required=True, help='Target IP')
#     PARSER.add_argument('-c', '--controller', default='TAHOE', help='Controller of CNEX SSD')
#     PARSER.add_argument('-f', '--flashtype', default='BICS', help='Flash type of CNEX SSD')
#     PARSER.add_argument('-L', '--loglevel', default='DEBUG',
#                         choices=('DEBUG', 'INFO', 'WARN', 'ERROR'), help='Log level')
#     GROUP = PARSER.add_mutually_exclusive_group(required=True)
#     GROUP.add_argument('-l', '--list', action='store_true', help='List test cases')
#     GROUP.add_argument('-n', '--nose', action='store_true', help='Run nosetests command')
#     GROUP.add_argument('-p', '--python', action='store_true', help='Run python command')
#     GROUP.add_argument('-r', '--regress', action='store_true', help='Regress nosetests file')
#     PARSER.add_argument('cmd', help='Command Line')
#     PARSER.add_argument('extra', nargs=argparse.REMAINDER)
#     ARGS = PARSER.parse_args()
#
#     sys.exit(run(ARGS))



# coding=utf-8
# pylint: disable=broad-except
from requests.exceptions import ConnectionError, ReadTimeout, ConnectTimeout

class State(object):
    FAIL = 0
    PASS = 1
    NOT_START = 2
    RUNNING = 3
    ABORT = 4

    ERROR_NOT_FOUND = 11
    ERROR_BASE_EXCEPTION = 12
    ERROR_TIMEOUT = 13
    ERROR_CONNECTION = 14
    ERROR_ABNORMAL_END = 15

    def __init__(self):
        pass


class RestError(Exception):
    def __init__(self, value):
        super(RestError, self).__init__()
        self.value = value

    def __str__(self):
        return repr(self.value)

def rest_get_call(full_url, session, timeout):
    """**REST Get Call**
    Makes a REST call using GET

    :param full_url: url of the rest call
    :type full_url: str
    :param session: requests session
    :type session: Session
    :param timeout: timeout in seconds of the call
    :type timeout: int
    :returns: returns results result
    :rtype: requests.Response

    """
    state_ = None
    msg_ = None
    try:
        result = session.get(full_url, timeout=timeout)
        if result.status_code == 200:
            return result.json()
        raise RestError(result.text)
    except (ReadTimeout, ConnectTimeout) as except_:
        msg_, state_ = except_, State.ERROR_TIMEOUT
    except ConnectionError as except_:
        msg_, state_ = except_, State.ERROR_CONNECTION
    except Exception as except_:
        msg_, state_ = except_, State.ERROR_BASE_EXCEPTION
    finally:
        result_ = {"resource": {"msg": str(msg_), "state": state_, "data": None}}
    return result_

def rest_post_json_call(full_url, session, key_value_set, timeout):
    """**REST Post Map Call**
    Makes a REST call using POST with a JSON Key value set

    :param full_url: url of the rest call
    :type full_url: str
    :param session: requests session
    :type session: Session
    :param key_value_set: KeyBuilder.get_json
    :type key_value_set: json
    :param timeout: timeout in seconds of the call
    :type timeout: int
    :param key_value_set:
    :returns: returns results result
    :rtype: requests.Response

    """
    state_ = None
    msg_ = None
    header = {'content-type': 'application/json'}
    try:
        result = session.post(full_url, data=key_value_set, headers=header, timeout=timeout)
        if result.status_code == 200:
            return result.json()
        raise RestError(result.text)
    except (ReadTimeout, ConnectTimeout) as except_:
        msg_, state_ = except_, State.ERROR_TIMEOUT
    except ConnectionError as except_:
        msg_, state_ = except_, State.ERROR_CONNECTION
    except Exception as except_:
        msg_, state_ = except_, State.ERROR_BASE_EXCEPTION
    finally:
        result_ = {"resource": {"msg": str(msg_), "state": state_, "data": None}}
    return result_


def rest_delete_call(full_url, session, timeout):
    """**REST Get Call**
    Makes a REST call using GET

    :param full_url: url of the rest call
    :type full_url: str
    :param session: requests session
    :type session: Session
    :param timeout: timeout in seconds of the call
    :type timeout: int
    :returns: returns results result
    :rtype: requests.Response

    """
    state_ = None
    msg_ = None
    try:
        result = session.delete(full_url, timeout=timeout)
        if result.status_code == 200:
            return result.json()
        raise RestError(result.text)
    except (ReadTimeout, ConnectTimeout) as except_:
        msg_, state_ = except_, State.ERROR_TIMEOUT
    except ConnectionError as except_:
        msg_, state_ = except_, State.ERROR_CONNECTION
    except Exception as except_:
        msg_, state_ = except_, State.ERROR_BASE_EXCEPTION
    finally:
        result_ = {"resource": {"msg": str(msg_), "state": state_, "data": None}}
    return result_



# coding=utf-8
# pylint: disable=broad-except, import-error
import pymysql as mysql



class SqlConnection(object):

    def __init__(self, host="172.29.129.8", port=3306, user="tester", passwd="Cnex!321", db_name="production_test"):
        self.host = host
        self.port = port
        self.user = user
        self.passwd = passwd
        self.db_name = db_name
        self.conn = mysql.connect(host=self.host, port=self.port, user=self.user,
                                  passwd=self.passwd, db=self.db_name, charset="utf8")
        self.cursor = self.conn.cursor()

    def __del__(self):
        self.cursor.close()
        self.conn.close()

    def get_result(self, key):
        cmd = "SELECT * FROM test_results WHERE test_key='{}'".format(key)
        self.cursor.execute(cmd)
        result = self.cursor.fetchone()
        return result



# coding=utf-8
# pylint: disable=import-error, broad-except
import time
from resources.models.remoter import RemoteCtrl


class RemoteResource(object):

    def __init__(self, host, port, session, state):
        self.host = host
        self.port = port
        self.testcase = session
        self.state = state
        cfg_file = './comConfig.yaml'
        self.remote = RemoteCtrl(host, ini=cfg_file)

    def list(self, filter_):
        '''
        get all test cases
        '''
        pass

    def run(self, test_case, mode="async"):
        """
        :param test_case: type string, test case name
        :param  mode: sync or async, run test case
        :return: type bool, return test case run result
        """
        for _ in range(5000):
            #ret = self.testcase.run(test_case, mode)
            #print(ret)
            #if self.check_test_status(ret['data'][0]):
            #    print('test failed')
            #    return 1
            ret = self.remote.reset_remote()
            print(ret)
            if ret['result']:
                print(ret['msg'])
                return 1
            if self.check_target_status():
                print('failed to connect server')
                return 1
        return 0

    def check_test_status(self, key):
        '''
        check test state through API
        '''
        while True:
            ret = self.testcase.get_async_result(key)
            print(ret)
            if 'state' in ret:
                if ret['state'] == 1:
                    return 0
                if ret['state'] not in [1, 2, 3, 13]:
                    return 1
            time.sleep(5)


    def check_target_status(self):
        '''
        check the target machine state through API
        '''
        fail_count = 1
        while fail_count <= 30:
            ret = self.state.get_state()
            if 'state' in ret:
                stat = ret['state']
                if stat == 1:
                    print("Server is Running\nIP:%s"%self.host)
                    return 0
                else:
                    print("Failed to connect Server IP:%s\n"%self.host)
                    fail_count += 1
                time.sleep(5)
        return 1



#!/usr/bin/python
'''
===============================

 Copyright (c) 2013 - CNEXLabs.
 All rights reserved.

 These coded instructions and program statements are
 copyrighted works and confidential proprietary information of CNEXLabs.
 They may not be modified, copied, reproduced, distributed, or disclosed to
 third parties in any manner, medium, or form, in whole or in part.

 Filename:              vRemoteCtrl.py
 Description:           N/A
 Change History:
 Date                Author                Comments
 2019/5/22         Liangmin Huang        Initial create.

===============================
'''
import os
import sys
import socket
import multiprocessing
import yaml

SOCK_TRANS_SIZE = 1024

class Client(object):
    def __init__(self, ip_addr, port, output):
        self.ip_addr = ip_addr
        self.port = port
        self.output = output
        self.lock = multiprocessing.Lock()
        self.pipe = multiprocessing.Pipe()

        self.main = multiprocessing.Process(target=self.client_process, args=(self.pipe[0], ))
        self.main.deamon = True
        self.main.start()

    def __del__(self):
        if "main" in dir(self):
            self.main.terminate()
            del self.main

    def lock_print(self, mesg, wrap=True):
        self.lock.acquire()
        try:
            if self.output is None:
                if wrap:
                    sys.stdout.write("%s\n" % mesg)
                else:
                    sys.stdout.write(mesg)
                sys.stdout.flush()
            else:
                self.output.append(mesg)
        finally:
            self.lock.release()

    def raw_input(self, mesg):
        self.lock_print(mesg, wrap=False)
        return raw_input()

    def client_process(self, pipe):
        conn = socket.socket()
        conn.connect((self.ip_addr, self.port))
        self.lock_print("[+] %s:%s link on!" % (self.ip_addr, self.port))

        try:
            while True:
                # get command from user
                cmd = pipe.recv()
                if cmd:
                    # send command to server
                    conn.send(cmd)

                    # get result
                    recv = conn.recv(SOCK_TRANS_SIZE)
                    if not recv:
                        raise StandardError("receive none!")
                    self.lock_print(recv)

                    # send result to client
                    pipe.send(recv)
        except BaseException:
            pass
        finally:
            conn.shutdown(socket.SHUT_RDWR)
            conn.close()
            self.lock_print("[-] recv link down!")

    def send(self, cmd):
        self.pipe[1].send(cmd)

    def recv(self):
        return self.pipe[1].recv()

    def is_alive(self):
        return self.main.is_alive()

    def close(self):
        self.__del__()

class RemoteCtrl(object):
    def __init__(self, host, ini, output=None):
        self.ini = ini
        self.output = output
        self.srv_port = 5000
        self.disable = self._read_cfg()
        self.host = host
    def _read_cfg(self):
        if not os.path.isfile(self.ini):
            print("[RemoteCtrl] No config file found, remote function will be disable")
            return 1
        filep = open(self.ini)
        self.config = yaml.load(filep)
        return 0
    def reset_remote(self):
        cmd = 'reset'
        #print "[vRemoteCtrl] cmd: %s, IP: %s"%(cmd,ip)
        return self._remote_ctrl(cmd, self.host)
    def power_off(self):
        cmd = 'off'
        return self._remote_ctrl(cmd, self.host)
    def power_on(self):
        cmd = 'on'
        return self._remote_ctrl(cmd, self.host)
    def _remote_ctrl(self, cmd, ip_addr):
        if self.disable:
            print("Remote function is disabled, Please check config file")
            return {'result':1, 'msg':'Remote function is disabled, Please check config file'}
        srv_ip = self._get_srv_ip(ip_addr)
        if not srv_ip:
            return {'result':1, 'msg':'Serve IP Not Found'}

        clt = Client(srv_ip, self.srv_port, self.output)
        cmd = "usbrelay %s %s"%(ip_addr, cmd)
        if clt.is_alive():
            clt.send(cmd)
            result = clt.recv()
            print(result)
        clt.close()
        return {'result':0, 'msg':'Operation suceed'}
    def _get_srv_ip(self, ip_addr):
        for srv in self.config:
            sip = srv['ip']
            clist = srv['client']
            for client in clist:
                if ip_addr in client['ip']:
                    return sip
        return 0



# coding=utf-8
# pylint: disable=wrong-import-position, relative-import
import sys
import os
import requests
sys.path.append(os.path.join(os.path.dirname(__file__)))
from resources.test_resource import TestResource
from resources.operation_resource import OperationResource
from resources.state_resource import StateResource
from resources.remote_resource import RemoteResource


class RestClient(object):

    def __init__(self, host, port=5000, time_out=5):
        __session = requests.Session()
        self.test = TestResource(host, port, __session, time_out=time_out)
        self.operation = OperationResource(host, port, __session, time_out=time_out)
        self.state = StateResource(host, port, __session, time_out=time_out)
        self.remote = RemoteResource(host, port, self.test, self.state)

if __name__ == '__main__':
    RC = RestClient("172.29.130.138", time_out=5)
    # print(RC.test.run("fio", "async"))
    print(RC.test.get_async_result("TQtRRRdCTe"))
    print(RC.test.get_test_lists("fio"))
    # #print(RC.state.get_state())
    # #print(RC.test_suite.get_test_lists("fio"))
    # # print(RC.test_case.list("fio"))
    # # print(RC.remote.run("test_fio_windows:TestFioWindows.test_rand_mix_rw_70_30", "async"))
    # print(RC.test_case.get_async_result("9cd1r0nbc2"))
    # print(RC.test_suite.get_async_result("9cd1r0nbc2"))
    # # print(RC.test_case.stop_tests())



import os
from ftplib import FTP


class FtpClient(object):

    def __init__(self, ip_addr, port, user, password, time_out=300):
        self.ip_addr = ip_addr
        self.port = port
        self.user = user
        self.password = password
        self.time_out = time_out
        self.ftp = FTP()
        self.connect()

    def __del__(self):
        self.ftp.quit()

    def connect(self):
        self.ftp.connect(self.ip_addr, self.port, self.time_out)
        self.ftp.login(self.user, self.password)

    def upload_binary_file(self, file_path):
        ret = False
        if os.path.exists(file_path):
            file_ = open(file_path, 'rb')
            file_name = os.path.basename(file_path)
            return_code = self.ftp.storbinary('STOR ' + file_name, file_)
            if "226" in return_code:
                ret = True
        else:
            print("upload file not exist: {}".format(file_path))
        return ret

if __name__ == '__main__':
    pass
    # try:
    #     Client = FtpClient("172.29.130.138", 10021, "ftp_user", "Cnex!321")
    #     ret = Client.upload_binary_file(r"d:\2.bin")
    #     print(ret)
    # except Exception as e:
    #     print(e)



# coding=utf-8
# pylint: disable=import-error
from resources.models.helper import rest_get_call


class StateResource(object):

    def __init__(self, host, port, session, time_out):
        self.host = host
        self.port = port
        self.session = session
        self.time_out = time_out

    def get_state(self):
        url_ = "http://{0}:{1}/state".format(self.host, self.port)
        result = rest_get_call(url_, self.session, self.time_out)
        return result["resource"]



from setuptools import setup, find_packages
 
setup(
    name = 'production_rest_client',
    version = '1.9.8',
    keywords = ['runner', 'client'],
    description = 'REST Runner',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
      ],
    license = 'MIT License',
	url = 'https://pypi.org/project/production_rest_client',
    install_requires = ['simplejson>=1.1'],
 
    author = 'weili.han',
    author_email = '826338726@qq.com',
    
    packages = find_packages(),
    platforms = 'any',
)


# coding=utf-8
# pylint: disable=import-error, broad-except
import os
import json
from resources.models.helper import rest_post_json_call
from resources.models.helper import State
from resources.models.ftp_client import FtpClient

class OperationResource(object):

    def __init__(self, host, port, session, time_out):
        self.host = host
        self.port = port
        self.session = session
        self.time_out = time_out

    def _upload_fw_bin(self, fw_path):
        ret = False
        try:
            ftp_client = FtpClient(self.host, 10021, "ftp_user", "Cnex!321")
            ret = ftp_client.upload_binary_file(fw_path)
        except Exception as exceptions:
            print(exceptions)
        return ret

    def upgrade(self, fw_path, device_index=1, slot=2):
        if self._upload_fw_bin(fw_path) is True:
            file_name = os.path.basename(fw_path)
            data = {"operate_name":"upgrade", "fw": file_name, "device_index":device_index, "slot":slot}
            url_ = "http://{0}:{1}/operation".format(self.host, self.port)
            result = rest_post_json_call(url_, self.session, json.dumps(data), self.time_out)
        else:
            result = dict()
            result["resource"] = {"data": None, "state": State.ERROR_BASE_EXCEPTION,
                                  "msg": "Send fw bin to test environment failed."}
        return result["resource"]



# coding=utf-8
# pylint: disable=import-error,eval-used
import json
from resources.models.helper import rest_get_call
from resources.models.helper import rest_post_json_call
from resources.models.helper import rest_delete_call
from resources.models.helper import State
from resources.models.database import SqlConnection


class TestResource(object):

    def __init__(self, host, port, session, time_out):
        self.host = host
        self.port = port
        self.session = session
        self.time_out = time_out

    def list(self, type_, filter_):
        """
        :param type_: test_case or test_suite
                filter_: string, filter_ condition
        :return: list, return list result
        """
        url_ = "http://{0}:{1}/test/{2}/{3}".format(self.host, self.port, type_, filter_)
        result = rest_get_call(url_, self.session, self.time_out)
        return result["resource"]

    def run(self, test_name, mode="sync"):
        """
        :param test_name: which test suite need to run
        :param  mode: sync or async, run test suite
        :return: list, return the test suite run results. include each test case in test suite
        """
        data = {"test": test_name, "mode": mode}
        url_ = "http://{0}:{1}/test".format(self.host, self.port)
        result = rest_post_json_call(url_, self.session, json.dumps(data), self.time_out)
        return result["resource"]

    def get_async_result(self, key):
        url_ = "http://{0}:{1}/test/results/{2}".format(self.host, self.port, key)
        result = rest_get_call(url_, self.session, self.time_out)
        if result["resource"]["state"] in [State.ERROR_TIMEOUT, State.ERROR_CONNECTION, State.ERROR_NOT_FOUND]:
            sql_connection = SqlConnection()
            ret = sql_connection.get_result(key)
            if ret:
                result["resource"] = {"msg": "Get result from database\n{}".format(ret[7]), "state": ret[4]}
        return result["resource"]

    def get_result_from_db(self, key):
        result = dict()
        sql_connection = SqlConnection()
        ret = sql_connection.get_result(key)
        if ret:
            result["resource"] = {"msg": ret[7], "state": ret[4]}
        else:
            result["resource"] = {"msg": "Database not find this test: {}".format(key), "state": State.ERROR_NOT_FOUND}

    def stop_tests(self, key=None):
        if key is None:
            url_ = "http://{0}:{1}/test".format(self.host, self.port)
        else:
            url_ = "http://{0}:{1}/test/{2}".format(self.host, self.port, key)
        result = rest_delete_call(url_, self.session, self.time_out)
        return result["resource"]

    def get_test_lists(self, test_suite_name):
        url_ = "http://{0}:{1}/test/testsuite/{2}".format(self.host, self.port, test_suite_name)
        result = rest_get_call(url_, self.session, self.time_out)
        if result["resource"]["state"] == State.PASS:
            result["resource"]["data"] = eval(result["resource"]["data"][0])
        return result["resource"]
